
# q-expansion of newform 230.4.a.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 230
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-2], [4], [-5], [3], [-2], [-38], [-45], [-74], [23], [283], [-303], [79], [-407], [-328], [360], [-561], [101], [-268], [-69], [-641], [994], [-884], [503], [1608], [1082], [879], [360], [1815], [-430], [255], [344], [-1348], [734], [2659], [-3196], [-100], [-787], [-70], [1784], [-28], [-32], [-1172], [-648], [-772], [-66], [-3358], [4489], [1990], [5524], [-1300], [-1212], [2761], [-5742], [-4162], [2464], [-4167], [-1639], [2655], [-826], [3930], [-7339], [-4763], [-4238], [4352], [-515], [-6816], [-8795], [-10654], [6624], [9719], [664], [5600], [5689], [-10626], [5160], [-11063], [-5374], [376], [-7790], [11909], [9162], [2086], [-9824], [-1793], [-7544], [6548], [14235], [-8039], [-9354], [-2402], [-14529], [1724], [-9920], [-8665], [-12673], [-3667], [-13246], [-4934], [17436], [-1714], [9616], [-21385], [-5967], [9714], [4244], [-15136], [22414], [22560], [-1240], [27931], [8660], [22326], [-7035], [-15808], [-9740], [-18730], [18191], [2126], [-27478], [-25430], [33610], [20732], [-24271], [-14280], [-4756], [5632], [-8536], [32137], [20329], [12879], [16689], [32328], [-13966], [-14103], [28805], [-26264], [29622], [-8129], [-35431], [-36025], [-35351], [-33534], [452], [-2379], [-599], [-16746], [-41754], [-8802], [7901], [-43358], [-5834], [-40940], [-22756], [20814], [-36661], [22340], [-34112], [-14133], [22442], [-36076], [-43316], [-8326], [47244], [-40460], [42369], [56331], [237], [-36856]]
